@@ -25,6 +25,7 @@
 #include "InputWindow.h"
 
 #include <string.h>
+#include <ctype.h>
 #include "version.h"
 #include <time.h>
 #include <X11/xpm.h>
@@ -47,14 +48,15 @@
 Window          inputWindow;
 int             iInputWindowX = INPUTWND_STARTX;
 int             iInputWindowY = INPUTWND_STARTY;
-int             iTempInputWindowX, iTempInputWindowY;	//¼ʱλãڹģʽ
 
 uint            iInputWindowHeight = INPUTWND_HEIGHT;
-uint            iFixedInputWindowWidth = 0;
+int		iFixedInputWindowWidth = 0;
 uint            iInputWindowWidth = INPUTWND_WIDTH;
 uint            iInputWindowUpWidth = INPUTWND_WIDTH;
 uint            iInputWindowDownWidth = INPUTWND_WIDTH;
 
+uint            INPUTWND_START_POS_UP = 8;
+
 MESSAGE_COLOR   messageColor[MESSAGE_TYPE_COUNT] = {
     {NULL, {0, 255 << 8, 0, 0}},
     {NULL, {0, 0, 0, 255 << 8}},
@@ -86,17 +88,22 @@ Bool            bShowNext = False;
 Bool            bTrackCursor = True;
 Bool            bCenterInputWindow = True;
 Bool            bShowInputWindowTriggering = True;
+Bool		bIsDisplaying = False;
 
 int             iCursorPos = 0;
 Bool            bShowCursor = False;
 
 _3D_EFFECT      _3DEffectInputWindow = _3D_LOWER;
 
+//GTK+ OverTheSpotʱ
+/* Issue 11: piaoairy: ΪӦgeneric_config_integer(), INT8 Ϊint */
+int		iOffsetX = 0;
+int		iOffsetY = 16;
+
 extern Display *dpy;
 extern int      iScreen;
 
 #ifdef _USE_XFT
-extern iconv_t  convUTF8;
 extern XftFont *xftFont;
 extern XftFont *xftFontEn;
 #else
@@ -115,6 +122,16 @@ extern Bool     bShowVersion;
 extern time_t   timeStart;
 extern uint     iHZInputed;
 
+extern CARD16	connect_id;
+
+extern int	iClientCursorX;
+extern int	iClientCursorY;
+
+#ifdef _DEBUG
+extern char     strUserLocale[];
+extern char     strXModifiers[];
+#endif
+
 Bool CreateInputWindow (void)
 {
     XSetWindowAttributes attrib;
@@ -141,8 +158,8 @@ Bool CreateInputWindow (void)
 	return False;
 
     XChangeWindowAttributes (dpy, inputWindow, attribmask, &attrib);
-    XSelectInput (dpy, inputWindow, ExposureMask | KeyPressMask | ButtonPressMask | ButtonReleaseMask | StructureNotifyMask | EnterWindowMask | PointerMotionMask | LeaveWindowMask | VisibilityChangeMask);
-
+    XSelectInput (dpy, inputWindow, ButtonPressMask | ButtonReleaseMask  | PointerMotionMask | ExposureMask);
+    
     InitInputWindowColor ();
 
     return True;
@@ -169,7 +186,10 @@ void DisplayInputWindow (void)
 #ifdef _DEBUG
     fprintf (stderr, "DISPLAY InputWindow\n");
 #endif
-    XMapRaised (dpy, inputWindow);
+    CalInputWindow();
+    MoveInputWindow(connect_id);
+    if (uMessageUp || uMessageDown)
+	XMapRaised (dpy, inputWindow);
 }
 
 void InitInputWindowColor (void)
@@ -179,6 +199,8 @@ void InitInputWindowColor (void)
     int             i;
 
     for (i = 0; i < MESSAGE_TYPE_COUNT; i++) {
+	if (messageColor[i].gc)
+	    XFreeGC (dpy, messageColor[i].gc);
 	messageColor[i].gc = XCreateGC (dpy, inputWindow, 0, &values);
 	if (XAllocColor (dpy, DefaultColormap (dpy, DefaultScreen (dpy)), &(messageColor[i].color)))
 	    iPixel = messageColor[i].color.pixel;
@@ -187,6 +209,8 @@ void InitInputWindowColor (void)
 	XSetForeground (dpy, messageColor[i].gc, iPixel);
     }
 
+    if (inputWindowLineColor.gc)
+	XFreeGC (dpy, inputWindowLineColor.gc);
     inputWindowLineColor.gc = XCreateGC (dpy, inputWindow, 0, &values);
     if (XAllocColor (dpy, DefaultColormap (dpy, DefaultScreen (dpy)), &(inputWindowLineColor.color)))
 	iPixel = inputWindowLineColor.color.pixel;
@@ -197,6 +221,9 @@ void InitInputWindowColor (void)
     cursorColor.color.red = cursorColor.color.red ^ inputWindowColor.backColor.red;
     cursorColor.color.green = cursorColor.color.green ^ inputWindowColor.backColor.green;
     cursorColor.color.blue = cursorColor.color.blue ^ inputWindowColor.backColor.blue;
+
+    if (cursorColor.gc)
+	XFreeGC (dpy, cursorColor.gc);
     cursorColor.gc = XCreateGC (dpy, inputWindow, 0, &values);
     //Ϊ˻ɫ
     if (XAllocColor (dpy, DefaultColormap (dpy, DefaultScreen (dpy)), &cursorColor.color))
@@ -214,35 +241,36 @@ void ResetInputWindow (void)
     uMessageUp = 0;
 }
 
-void DrawInputWindow (void)
+void CalInputWindow (void)
 {
     int             i;
-    XImage         *mask;
-    XpmAttributes   attrib;
 
 #ifdef _USE_XFT
     char            strTemp[MESSAGE_MAX_LENGTH];
     char           *p1, *p2;
     Bool            bEn;
 #endif
-    XWindowAttributes wa;
 
 #ifdef _DEBUG
-    fprintf (stderr, "DRAW InputWindow\n");
+    fprintf (stderr, "CAL InputWindow\n");
 #endif
 
-    XClearArea (dpy, inputWindow, 2, 2, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
-    XClearArea (dpy, inputWindow, 2, iInputWindowHeight / 2 + 1, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
-
     if (!uMessageUp && !uMessageDown) {
 	bShowCursor = False;
-
 	if (bShowVersion) {
 	    uMessageUp = 1;
 	    strcpy (messageUp[0].strMsg, "FCITX ");
 	    strcat (messageUp[0].strMsg, FCITX_VERSION);
 	    messageUp[0].type = MSG_TIPS;
 	}
+
+#ifdef _DEBUG
+	uMessageDown = 1;
+	strcpy (messageDown[0].strMsg, strUserLocale);
+	strcat (messageDown[0].strMsg, " - ");
+	strcat (messageDown[0].strMsg, strXModifiers);
+	messageDown[0].type = MSG_CODE;
+#else
 	//ʾٶ
 	if (bStartRecordType && bShowUserSpeed) {
 	    double          timePassed;
@@ -265,13 +293,7 @@ void DrawInputWindow (void)
 	    sprintf (messageDown[5].strMsg, "%u", iHZInputed);
 	    messageDown[5].type = MSG_CODE;
 	}
-	else {
-	    if (bShowVersion) {
-		uMessageDown = 1;
-		strcpy (messageDown[0].strMsg, "http://www.fcitx.org");
-		messageDown[0].type = MSG_CODE;
-	    }
-	}
+#endif
     }
 
     iInputWindowUpWidth = 2 * INPUTWND_START_POS_UP + 1;
@@ -360,28 +382,16 @@ void DrawInputWindow (void)
 	if (iInputWindowWidth < iFixedInputWindowWidth)
 	    iInputWindowWidth = iFixedInputWindowWidth;
     }
+}
 
-    XGetWindowAttributes (dpy, inputWindow, &wa);
-    if ((wa.x + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
-	i = DisplayWidth (dpy, iScreen) - iInputWindowWidth - 2;
-    else if (wa.x < 0) {
-	if (iInputWindowWidth <= DisplayWidth (dpy, iScreen))
-	    i = 0;
-	else
-	    i = DisplayWidth (dpy, iScreen) - iInputWindowWidth;
-    }
-    else
-	i = wa.x;
-
-    XMoveWindow (dpy, inputWindow, i, wa.y);
-    if (bCenterInputWindow && !bTrackCursor) {
-	iInputWindowX = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
-	if (iInputWindowX < 0)
-	    iInputWindowX = 0;
-	XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
-    }
-
-    XResizeWindow (dpy, inputWindow, iInputWindowWidth, iInputWindowHeight);
+void DrawInputWindow(void)
+{
+    XImage         *mask;
+    XpmAttributes   attrib;
+    int	i;
+    
+    XClearArea (dpy, inputWindow, 2, 2, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
+    XClearArea (dpy, inputWindow, 2, iInputWindowHeight / 2 + 1, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
 
     DisplayMessageUp ();
     DisplayMessageDown ();
@@ -395,7 +405,6 @@ void DrawInputWindow (void)
 	Draw3DEffect (inputWindow, 0, 0, iInputWindowWidth, iInputWindowHeight, _3D_LOWER);
 
     XDrawRectangle (dpy, inputWindow, inputWindowLineColor.gc, 0, 0, iInputWindowWidth - 1, iInputWindowHeight - 1);
-    //XDrawRectangle (dpy, inputWindow, inputWindowLineColor.gc, 1, 1, iInputWindowWidth - 3, iInputWindowHeight - 3);
     if (_3DEffectInputWindow == _3D_LOWER)
 	XDrawLine (dpy, inputWindow, lightGC, 2 + 5, iInputWindowHeight / 2 - 1, iInputWindowWidth - 2 - 5, iInputWindowHeight / 2 - 1);
     else if (_3DEffectInputWindow == _3D_UPPER)
@@ -430,7 +439,7 @@ void DrawInputWindow (void)
 void DisplayMessageUp (void)
 {
     int             i = 0;
-    int             iPos;
+    int             iPos = 0;
     int             iCursorPixPos = 0;
     int             iChar;
     char            strText[MESSAGE_MAX_LENGTH];
@@ -443,12 +452,13 @@ void DisplayMessageUp (void)
 
     char           *strGBKT;
 
-    iPos = INPUTWND_START_POS_UP;
+    iInputWindowUpWidth = INPUTWND_START_POS_UP;
     iChar = iCursorPos;
 
     for (i = 0; i < uMessageUp; i++) {
 #ifdef _USE_XFT
 	p1 = messageUp[i].strMsg;
+	iPos = 0;
 	while (*p1) {
 	    p2 = strTemp;
 	    if (isprint (*p1))	//ʹ
@@ -475,9 +485,8 @@ void DisplayMessageUp (void)
 
 	    strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (strTemp) : strTemp;
 
-	    iInputWindowUpWidth = StringWidth (strGBKT, (bEn) ? xftFontEn : xftFont);
-	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strGBKT, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].color);
-	    iPos += iInputWindowUpWidth;
+	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strGBKT, iInputWindowUpWidth + iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].color);
+	    iPos += StringWidth (strGBKT, (bEn) ? xftFontEn : xftFont);
 
 	    if (bUseGBKT)
 		free (strGBKT);
@@ -485,14 +494,13 @@ void DisplayMessageUp (void)
 #else
 	strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (messageUp[i].strMsg) : messageUp[i].strMsg;
 
-	iInputWindowUpWidth = StringWidth (strGBKT, fontSet);
-	OutputString (inputWindow, fontSet, strGBKT, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].gc);
-	iPos += iInputWindowUpWidth;
+	OutputString (inputWindow, fontSet, strGBKT, iInputWindowUpWidth, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].gc);
+	iPos = StringWidth (strGBKT, fontSet);
 
 	if (bUseGBKT)
 	    free (strGBKT);
-
 #endif
+	iInputWindowUpWidth += iPos;
 
 	if (bShowCursor && iChar) {
 	    if (strlen (messageUp[i].strMsg) > iChar) {
@@ -532,7 +540,7 @@ void DisplayMessageUp (void)
 		iChar = 0;
 	    }
 	    else {
-		iCursorPixPos += iInputWindowUpWidth;
+		iCursorPixPos += iPos;
 		iChar -= strlen (messageUp[i].strMsg);
 	    }
 	}
@@ -598,10 +606,6 @@ void DisplayMessageDown (void)
 	    if (bUseGBKT)
 		free (strGBKT);
 	}
-
-	/*iInputWindowDownWidth = StringWidth (messageDown[i].strMsg, xftFont);
-	   OutputString (inputWindow, xftFont, messageDown[i].strMsg, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].color);
-	   iPos += iInputWindowDownWidth; */
 #else
 	strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (messageDown[i].strMsg) : messageDown[i].strMsg;
 
@@ -620,3 +624,45 @@ void DrawCursor (int iPos)
     XDrawLine (dpy, inputWindow, cursorColor.gc, iPos, 8, iPos, iInputWindowHeight / 2 - 4);
     XDrawLine (dpy, inputWindow, cursorColor.gc, iPos + 1, 8, iPos + 1, iInputWindowHeight / 2 - 4);
 }
+
+void MoveInputWindow(CARD16 connect_id)
+{
+    if (ConnectIDGetTrackCursor (connect_id) && bTrackCursor) {
+        int iTempInputWindowX, iTempInputWindowY;
+
+	if (iClientCursorX < 0)
+	    iTempInputWindowX = 0;
+	else 
+	    iTempInputWindowX = iClientCursorX + iOffsetX;
+	    
+	if (iClientCursorY < 0)
+	    iTempInputWindowY = 0;
+	else
+	    iTempInputWindowY = iClientCursorY + iOffsetY;	
+
+	if ((iTempInputWindowX + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
+	    iTempInputWindowX = DisplayWidth (dpy, iScreen) - iInputWindowWidth;
+
+	if ((iTempInputWindowY + iInputWindowHeight) > DisplayHeight (dpy, iScreen)) {
+	    if ( iTempInputWindowY > DisplayHeight (dpy, iScreen) )
+	        iTempInputWindowY = DisplayHeight (dpy, iScreen) - 2 * iInputWindowHeight;
+	    else
+	        iTempInputWindowY = iTempInputWindowY - 2 * iInputWindowHeight;
+	}
+
+	XMoveResizeWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY, iInputWindowWidth, iInputWindowHeight);  
+	ConnectIDSetPos (connect_id, iTempInputWindowX - iOffsetX, iTempInputWindowY - iOffsetX);
+    }
+    else {
+	position * pos = ConnectIDGetPos(connect_id);
+	if (bCenterInputWindow) {
+	    iInputWindowX = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
+	    if (iInputWindowX < 0)
+		iInputWindowX = 0;
+	}
+	else
+	    iInputWindowX = pos ? pos->x : iInputWindowX;
+
+	XMoveResizeWindow (dpy, inputWindow, iInputWindowX, pos ? pos->y : iInputWindowY, iInputWindowWidth, iInputWindowHeight);  
+    }
+}
