@@ -17,17 +17,28 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+/**
+ * @file   InputWindow.h
+ * @author Yuking yuking_net@sohu.com
+ * @date   2008-1-16
+ * 
+ * @brief  
+ * 
+ * 
+ */
+
 #ifndef _INPUT_WINDOW_H
 #define _INPUT_WINDOW_H
 
 #include <X11/Xlib.h>
+#include "IMdkit.h"
 
 #define INPUTWND_STARTX	300
 #define INPUTWND_WIDTH	50
 #define INPUTWND_STARTY	420
 #define INPUTWND_HEIGHT	40
 
-#define INPUTWND_START_POS_UP	8
+/* #define INPUTWND_START_POS_UP	8 */
 #define INPUTWND_START_POS_DOWN	8
 
 #define MESSAGE_MAX_LENGTH	300	//ʾȣַ
@@ -52,12 +63,13 @@ typedef struct {
 
 Bool            CreateInputWindow (void);
 void            DisplayInputWindow (void);
+void		DrawInputWindow (void);
+void		CalInputWindow (void);
 void            InitInputWindowColor (void);
 void            CalculateInputWindowHeight (void);
 void            DrawCursor (int iPos);
 void            DisplayMessageUp (void);
 void            DisplayMessageDown (void);
-void            DrawInputWindow (void);
 void            ResetInputWindow (void);
-
+void		MoveInputWindow(CARD16 connect_id);
 #endif
