@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 #include <signal.h>
+#include <limits.h>
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -43,9 +44,11 @@
 #define SIGUNUSED 32
 #endif
 
-XErrorHandler   oldXErrorHandler;
+#ifdef _ENABLE_LOG
+FILE *logfile = (FILE *)NULL;
+#endif
 
-//extern Bool     bLumaQQ;
+XErrorHandler   oldXErrorHandler;
 
 void SetMyExceptionHandler (void)
 {
@@ -56,33 +59,50 @@ void SetMyExceptionHandler (void)
 }
 
 void OnException (int signo)
-{
-
-    fprintf (stdout, "\nFCITX -- Get Signal No.: %d\n", signo);
+{   
+#ifdef _ENABLE_LOG
+    char    buf[PATH_MAX], *pbuf;
+    struct tm  *ts;
+    time_t now;
 
+    if ( !logfile ) {
+	pbuf = getenv("HOME");		// ӻлȡǰûĿ¼ľ·
+        if(pbuf) {
+	    snprintf(buf, PATH_MAX, "%s/.fcitx/fcitx.log", pbuf);
+            logfile=fopen(buf, "wt");
+        }
+    }
+    
+    if ( logfile ) {
+	now=time(NULL);
+	ts = localtime(&now);
+	strftime(buf, sizeof(buf), "%a %Y-%m-%d %H:%M:%S %Z", ts);
+	fprintf (logfile, "FCITX -- Get Signal No.: %d (%s)\n", signo,buf);
+    }
+#endif
+    
+    fprintf (stderr, "\nFCITX -- Get Signal No.: %d\n", signo);
+    
+    if ( signo!=SIGSEGV && signo!=SIGCONT)
+        SaveIM();
+    
     switch (signo) {
-	/* SIGSEGVԼ⣬ʱִбܻ뷨ļԲܵSaveIM () */
-    case SIGSEGV:
-    case SIGFPE:
-	break;
     case SIGHUP:
 	LoadConfig (False);
 	SetIM ();
 	break;
     case SIGINT:
-    	SaveIM ();
-    	exit (1);
-    case SIGUSR1:
-    case SIGCHLD:
-    case SIGWINCH:
-    case SIGTTIN:
-    case SIGTSTP:	
-    case SIGSTOP:
     case SIGTERM:
-    case SIGQUIT:
-    case SIGKILL:
+    case SIGPIPE:
+    case SIGSEGV:
+#ifdef _ENABLE_LOG    
+        if ( !logfile ) {
+            fclose(logfile);
+            logfile = (FILE *)NULL;
+        }
+#endif
+	exit (0);
     default:
-    	SaveIM ();
 	break;
     }
 }
@@ -94,14 +114,40 @@ void SetMyXErrorHandler (void)
 
 int MyXErrorHandler (Display * dpy, XErrorEvent * event)
 {
-    /*Ϣûκôȥ
-       char            str[256];
+#ifdef _ENABLE_LOG
+    char    str[256];
+    char    buf[PATH_MAX], *pbuf;
+    struct tm  *ts;
+    time_t now;
 
-       XGetErrorText (dpy, event->error_code, str, 255);
-       fprintf (stdout, "fcitx: %s\n", str);
-     */
-    if (event->error_code != 3 && event->error_code != BadMatch)	// xterm will generate 3
+    if ( !logfile ) {
+        pbuf = getenv("HOME");		// ӻлȡǰûĿ¼ľ·
+        if(pbuf) {
+            snprintf(buf, PATH_MAX, "%s/.fcitx/fcitx.log", pbuf);
+            logfile = fopen(buf, "wt");
+	    if ( logfile ) {
+                now = time(NULL);
+	        ts = localtime(&now);
+	        strftime(buf, sizeof(buf), "%a %Y-%m-%d %H:%M:%S %Z", ts);
+	        XGetErrorText (dpy, event->error_code, str, 255);
+	        fprintf (logfile, "fcitx: %s\n", buf);
+	        fprintf (logfile, "fcitx: %s\n", str);
+	     }
+        }
+    }
+#endif
+
+    SaveIM();
+
+    if (event->error_code != 3 && event->error_code != BadMatch) {	// xterm will generate 3
+#ifdef _ENABLE_LOG
+	if ( !logfile ) {
+            fclose(logfile);
+            logfile = (FILE *)NULL;
+        }
+#endif	
 	oldXErrorHandler (dpy, event);
+    }
 
     return 0;
 }
