@@ -46,6 +46,11 @@
 #include "ui.h"
 #include "MainWindow.h"
 #include "InputWindow.h"
+
+#ifdef _ENABLE_TRAY
+#include "TrayWindow.h"
+#endif
+
 #include "vk.h"
 #include "ime.h"
 #include "table.h"
@@ -150,9 +155,9 @@ int main (int argc, char *argv[])
     //ڵʾ
     if (hideMainWindow != HM_HIDE) {
 	DisplayMainWindow ();
-	DrawMainWindow ();
+	DrawMainWindow ();	
     }
-
+    
     //ʼ뷨
     if (!InitXIM (inputWindow, imname))
 	exit (4);
@@ -172,6 +177,11 @@ int main (int argc, char *argv[])
     
     SetMyExceptionHandler();		//¼
 
+#ifdef _ENABLE_TRAY
+    CreateTrayWindow ();		//ϵͳ̴
+    DrawTrayWindow (INACTIVE_ICON);	//ʾͼ
+#endif
+    
     //ѭXWindowϢѭ
     for (;;) {
 	XNextEvent (dpy, &event);					//ȴһ¼
