@@ -46,9 +46,6 @@ uint            iCounter = 0;
 Bool            bPYBaseDictLoaded = False;
 Bool            bPYOtherDictLoaded = False;
 
-/*
-Bool            bSingleHZMode = False;	//£ֻе룬дƥ䡢ɼ棬뷨еƴ
-*/
 PyFreq         *pyFreq = NULL, *pCurFreq = NULL;
 uint            iPYFreqCount = 0;
 
@@ -127,17 +124,8 @@ Bool LoadPYBaseDict (void)
 {
     FILE           *fp;
     char            strPath[PATH_MAX];
-
-/*
-#if defined(DARWIN)
-    unsigned int    i, j, iLen;
-#else*/
     int             i, j, iLen;
 
-/*
-#endif
-*/
-
     strcpy (strPath, PKGDATADIR "/data/");
     strcat (strPath, PY_BASE_FILE);
     fp = fopen (strPath, "rb");
@@ -201,21 +189,12 @@ Bool LoadPYOtherDict (void)
 	while (!feof (fp)) {
 	    if (!fread (&i, sizeof (int), 1, fp))
 		break;
-/*
-#if defined(DARWIN)
-            i = ReverseInt (i);
-#endif
-*/
 	    if (!fread (strBase, sizeof (char) * 2, 1, fp))
 		break;
 	    strBase[2] = '\0';
 	    if (!fread (&k, sizeof (int), 1, fp))
 		break;
-/*
-#if defined(DARWIN)
-            k = ReverseInt (k);
-#endif
-*/
+
 	    j = GetBaseIndex (i, strBase);
 	    if (j == -1)
 		break;
@@ -453,7 +432,6 @@ void ResetPYStatus ()
     bIsPYAddFreq = False;
     bIsPYDelFreq = False;
     bIsPYDelUserPhr = False;
-    //bSingleHZMode = False;
 
     findMap.iMode = PARSE_SINGLEHZ;	//ֻҪPARSE_ERRORͿ
 }
@@ -479,7 +457,6 @@ INPUT_RETURN_VALUE DoPYInput (int iKey)
 
     if (!bPYBaseDictLoaded)
 	LoadPYBaseDict ();
-    //if (!bSingleHZMode && !bPYOtherDictLoaded)
     if (!bPYOtherDictLoaded)
 	LoadPYOtherDict ();
 
@@ -928,7 +905,6 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 {
     int             iVal;
 
-    //if (findMap.iMode == PARSE_ERROR || (bSingleHZMode && findMap.iHZCount > 1)) {
     if (findMap.iMode == PARSE_ERROR) {
 	uMessageDown = 0;
 	iCandPageCount = 0;
@@ -956,7 +932,6 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 	    pCurFreq = pCurFreq->next;
 	}
 
-	//if (!bSingleHZMode && bPYCreateAuto)
 	if (bPYCreateAuto)
 	    PYCreateAuto ();
     }
@@ -983,7 +958,6 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
     }
 
     if (!(pCurFreq && pCurFreq->bIsSym)) {
-	//if (!iCurrentCandPage && strPYAuto[0] && !bSingleHZMode) {
 	if (!iCurrentCandPage && strPYAuto[0]) {
 	    iCandWordCount = 1;
 	    PYCandWords[0].iWhich = PY_CAND_AUTO;
@@ -1471,11 +1445,11 @@ char           *PYGetCandWord (int iIndex)
 
     if (pIndex && (*pIndex != iCounter))
 	*pIndex = ++iCounter;
-    if (iOrderCount == AUTOSAVE_ORDER_COUNT) {
+    if (iOrderCount >= AUTOSAVE_ORDER_COUNT) {
 	SavePYIndex ();
 	iOrderCount = 0;
     }
-    if (iNewFreqCount == AUTOSAVE_FREQ_COUNT) {
+    if (iNewFreqCount >= AUTOSAVE_FREQ_COUNT) {
 	SavePYFreq ();
 	iNewFreqCount = 0;
     }
@@ -1500,7 +1474,6 @@ char           *PYGetCandWord (int iIndex)
 	    strcat (strHZString, pBaseMap);
 	if (pPhraseMap)
 	    strcat (strHZString, pPhraseMap);
-	//if (!bSingleHZMode && bAddNewPhrase && (strlen (strPYAuto) <= (MAX_PY_PHRASE_LENGTH * 2)))
 	if (bAddNewPhrase && (strlen (strPYAuto) <= (MAX_PY_PHRASE_LENGTH * 2)))
 	    PYAddUserPhrase (strPYAuto, strHZString);
 	uMessageDown = 0;
@@ -1551,14 +1524,11 @@ void PYGetCandWordsForward (void)
 
 void PYGetCandWordsBackward (void)
 {
-    //if (pCurFreq && pCurFreq->bIsSym && !bSingleHZMode)
     if (pCurFreq && pCurFreq->bIsSym)
 	PYGetSymCandWords (SM_PREV);
     else {
-	//if (!bSingleHZMode)
 	PYGetFreqCandWords (SM_PREV);
 	PYGetBaseCandWords (SM_PREV);
-	//if (iCandWordCount == iMaxCandWord || bSingleHZMode)
 	if (iCandWordCount == iMaxCandWord)
 	    return;
 	PYGetPhraseCandWords (SM_PREV);
@@ -2674,19 +2644,17 @@ void SavePYIndex (void)
 
     //ٱΪ0ϵͳ
     for (i = 0; i < iPYFACount; i++) {
-	if (PYFAList[i].iChangeCount) {
-	    for (j = 0; j < PYFAList[i].iBase; j++) {
-		if (PYFAList[i].pyBase[j].iChangeCount) {
-		    for (k = 0; k < PYFAList[i].pyBase[j].iPhrase; k++) {
-			if (PYFAList[i].pyBase[j].phrase[k].iIndex) {
-			    fwrite (&i, sizeof (int), 1, fp);
-			    fwrite (&j, sizeof (int), 1, fp);
-			    fwrite (&k, sizeof (int), 1, fp);
-			    l = PYFAList[i].pyBase[j].phrase[k].iIndex;
-			    fwrite (&l, sizeof (uint), 1, fp);
-			    l = PYFAList[i].pyBase[j].phrase[k].iHit;
-			    fwrite (&l, sizeof (uint), 1, fp);
-			}
+	for (j = 0; j < PYFAList[i].iBase; j++) {
+	    if (PYFAList[i].pyBase[j].iChangeCount) {
+		for (k = 0; k < PYFAList[i].pyBase[j].iPhrase; k++) {
+		    if (PYFAList[i].pyBase[j].phrase[k].iIndex) {
+			fwrite (&i, sizeof (int), 1, fp);
+			fwrite (&j, sizeof (int), 1, fp);
+			fwrite (&k, sizeof (int), 1, fp);
+			l = PYFAList[i].pyBase[j].phrase[k].iIndex;
+			fwrite (&l, sizeof (uint), 1, fp);
+			l = PYFAList[i].pyBase[j].phrase[k].iHit;
+			fwrite (&l, sizeof (uint), 1, fp);
 		    }
 		}
 	    }
@@ -3065,46 +3033,3 @@ void PYGetPYByHZ (char *strHZ, char *strPY)
 	}
     }
 }
-
-/*
-void PP ()
-{
-    int             iVal;
-    char           *pBase = NULL;
-    char           *pPhrase = NULL;
-    char            ss[1000];
-
-    for (iVal = 0; iVal < iCandWordCount; iVal++) {
-	if (PYCandWords[iVal].iWhich == PY_CAND_AUTO)
-	    strcpy (ss, strPYAuto);
-	else {
-	    pPhrase = NULL;
-	    switch (PYCandWords[iVal].iWhich) {
-	    case PY_CAND_BASE:	//ϵͳ
-		pBase = PYFAList[PYCandWords[iVal].cand.base.iPYFA].pyBase[PYCandWords[iVal].cand.base.iBase].strHZ;
-		break;
-	    case PY_CAND_SYMPHRASE:	//ϵͳ
-		pBase = PYFAList[PYCandWords[iVal].cand.phrase.iPYFA].pyBase[PYCandWords[iVal].cand.phrase.iBase].strHZ;
-		pPhrase = PYCandWords[iVal].cand.phrase.phrase->strPhrase;
-		break;
-	    case PY_CAND_USERPHRASE:	//û
-		pBase = PYFAList[PYCandWords[iVal].cand.phrase.iPYFA].pyBase[PYCandWords[iVal].cand.phrase.iBase].strHZ;
-		pPhrase = PYCandWords[iVal].cand.phrase.phrase->strPhrase;
-		break;
-	    case PY_CAND_FREQ:	//ǳ
-		pBase = PYCandWords[iVal].cand.freq.hz->strHZ;
-		break;
-	    case PY_CAND_SYMBOL:	//
-		pBase = PYCandWords[iVal].cand.freq.hz->strHZ;
-		break;
-	    }
-	    strcpy (ss, pBase);
-	    if (pPhrase)
-		strcat (ss, pPhrase);
-	}
-
-	printf ("%d: %s\n", iVal, ss);
-    }
-    printf ("---------------------\n");
-}
-*/
