@@ -58,7 +58,7 @@ Bool            MapPY (char *strPY, char strMap[3], PYPARSEINPUTMODE mode);
 
 Bool            MapToPY (char strMap[3], char *strPY);
 int             CmpMap (char *strMap1, char *strMap2, int *iMatchedLength);
-int             Cmp1Map (char map1, char map2, Bool b);
+int		Cmp1Map (char map1, char map2, Bool b, Bool bUseMH);
 int             Cmp2Map (char map1[3], char map2[3]);
 
 #endif
