@@ -104,8 +104,8 @@ SP_S            SPMap_S[] = {
 Bool            bSP_UseSemicolon = False;
 Bool            bSP = False;
 char            cNonS = 'o';
-char	    	strDefaultSP[100]="Ȼ";
-SP_FROM	        iSPFrom=SP_FROM_USER;	//Ӻδȡ˫ƴ0ûĿ¼1ϵͳã2ϵͳ˫ƴļ
+char            strDefaultSP[100] = "Ȼ";
+SP_FROM         iSPFrom = SP_FROM_USER;	//Ӻδȡ˫ƴ0ûĿ¼1ϵͳã2ϵͳ˫ƴļ
 
 //extern Bool     bSingleHZMode;
 
@@ -123,15 +123,15 @@ void LoadSPData (void)
     char            strPath[PATH_MAX];
     char            str[100], strS[5], *pstr;
     int             i;
-    Bool	    bIsDefault=False;
-    Bool	    bIsFromSystemSPConfig=False;
+    Bool            bIsDefault = False;
+    Bool            bIsFromSystemSPConfig = False;
 
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/");
 
     if (access (strPath, 0))
 	mkdir (strPath, S_IRWXU);
-    
+
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/");
     strcat (strPath, "sp.dat");
@@ -140,7 +140,7 @@ void LoadSPData (void)
 	strcpy (strPath, PKGDATADIR "/data/");
 	strcat (strPath, "sp.dat");
 	bIsFromSystemSPConfig = True;
-	if (iSPFrom != SP_FROM_SYSTEM_CONFIG )
+	if (iSPFrom != SP_FROM_SYSTEM_CONFIG)
 	    iSPFrom = SP_FROM_SYSTEM_SP_CONFIG;
     }
 
@@ -149,43 +149,44 @@ void LoadSPData (void)
     if (!fp)
 	return;
 
-    if ( !bIsFromSystemSPConfig )
+    if (!bIsFromSystemSPConfig)
 	iSPFrom = SP_FROM_USER;
-    
+
     while (1) {
 	if (!fgets (str, 100, fp))
 	    break;
 
 	i = strlen (str) - 1;
-	while (str[i] == ' ' || str[i] == '\n')
+	while ((i >= 0) && (str[i] == ' ' || str[i] == '\n'))
 	    str[i--] = '\0';
+
 	pstr = str;
 	if (*pstr == ' ' || *pstr == '\t')
 	    pstr++;
 	if (!strlen (pstr) || pstr[0] == '#')
 	    continue;
-	
-	if (!strncmp(pstr,"ĬϷ=",9)) {
-		if (iSPFrom!=SP_FROM_SYSTEM_CONFIG) {
-		    pstr+=9;
-		    if (*pstr == ' ' || *pstr == '\t')
-			pstr++;
-		    strcpy(strDefaultSP,pstr);
-		}
-		continue;
-	}
-	
-	if (!strncmp(pstr,"=",9)) {
-		pstr+=9;
+
+	if (!strncmp (pstr, "ĬϷ=", 9)) {
+	    if (iSPFrom != SP_FROM_SYSTEM_CONFIG) {
+		pstr += 9;
 		if (*pstr == ' ' || *pstr == '\t')
-			pstr++;
-		bIsDefault = !(strcmp(strDefaultSP,pstr));
-		continue;
+		    pstr++;
+		strcpy (strDefaultSP, pstr);
+	    }
+	    continue;
 	}
-	
-	if ( !bIsDefault )
-		continue;
-	
+
+	if (!strncmp (pstr, "=", 9)) {
+	    pstr += 9;
+	    if (*pstr == ' ' || *pstr == '\t')
+		pstr++;
+	    bIsDefault = !(strcmp (strDefaultSP, pstr));
+	    continue;
+	}
+
+	if (!bIsDefault)
+	    continue;
+
 	if (pstr[0] == '=')	//ĸ
 	    cNonS = tolower (pstr[1]);
 	else {
