@@ -52,8 +52,6 @@ extern int      iMainWindowX;
 extern int      iMainWindowY;
 extern int      iInputWindowX;
 extern int      iInputWindowY;
-extern int      iTempInputWindowX;
-extern int      iTempInputWindowY;
 extern int      iInputWindowWidth;
 extern int      iInputWindowHeight;
 
@@ -89,6 +87,8 @@ extern HOTKEYS  hkTrack[];
 extern HOTKEYS  hkGetPY[];
 extern HOTKEYS  hkGBT[];
 extern HOTKEYS	hkHideMainWindow[];
+extern HOTKEYS	hkSaveAll[];
+extern HOTKEYS	hkVK[];
 
 extern KEY_CODE switchKey;
 extern XIMTriggerKey *Trigger_Keys;
@@ -155,7 +155,6 @@ extern Bool     bUseTable;
 extern char     strDefaultSP[];
 extern SP_FROM  iSPFrom;
 
-//extern Bool     bLumaQQ;
 extern char     cPYYCDZ[];
 extern char	strExternIM[];
 
@@ -176,27 +175,9 @@ extern Bool     bUseBold;
 extern int      iOffsetX;
 extern int      iOffsetY;
 
-/*
-extern Bool     bStaticXIM;
-extern Bool     bNoReleaseKey;
-*/
-
-/*
-#if defined(DARWIN)*/
-/* function to reverse byte order for integer
-this is required for Mac machine*/
-/*int ReverseInt (unsigned int pc_int)
-{
-    int             mac_int;
-    unsigned char  *p;
-
-    mac_int = pc_int;
-    p = (unsigned char *) &pc_int;
-    mac_int = (p[3] << 24) + (p[2] << 16) + (p[1] << 8) + p[0];
-    return mac_int;
-}
+#ifdef _ENABLE_TRAY
+extern Bool	bUseTrayIcon;
 #endif
-*/
 
 Bool MyStrcmp (char *str1, char *str2)
 {
@@ -449,6 +430,17 @@ inline static int hide_main_window(Configure *c, void *a, int isread)
     return 0;
 }
 
+/* л */
+inline static int switch_vk(Configure *c, void *a, int isread)
+{
+    if(isread)
+        SetHotKey((char *)a, hkVK);
+    else
+        fprintf((FILE *)a, "%s=%s\n", c->name, "CTRL_ALT_K");
+
+    return 0;
+}
+
 /* GBK֧ */
 inline static int gbk_support(Configure *c, void *a, int isread)
 {
@@ -566,6 +558,17 @@ inline static int second_third_candidate_word(Configure *c, void *a, int isread)
     return 0;
 }
 
+/* ʿ */
+inline static int save_all(Configure *c, void *a, int isread)
+{
+    if(isread)
+        SetHotKey((char *)a, hkSaveAll);
+    else
+        fprintf((FILE *)a, "%s=%s\n", c->name, "CTRL_ALT_S");
+
+    return 0;
+}
+
 /* Ĭ˫ƴ */
 inline static int default_shuangpin_scheme(Configure *c, void *a, int isread)
 {
@@ -590,17 +593,6 @@ inline static int add_pinyin_frequently_used_word(Configure *c, void *a, int isr
     return 0;
 }
 
-/* ɾƴ */
-inline static int delete_pinyin_frequently_used_word(Configure *c, void *a, int isread)
-{
-    if(isread)
-        SetHotKey((char *)a, hkPYDelFreq);
-    else
-        fprintf((FILE *)a, "%s=%s\n", c->name, "CTRL_7");
-
-    return 0;
-}
-
 /* ɾƴû */
 inline static int delete_pinyin_user_create_phrase(Configure *c, void *a, int isread)
 {
@@ -612,6 +604,17 @@ inline static int delete_pinyin_user_create_phrase(Configure *c, void *a, int is
     return 0;
 }
 
+/* ɾƴ */
+inline static int delete_pinyin_frequently_used_word(Configure *c, void *a, int isread)
+{
+    if(isread)
+        SetHotKey((char *)a, hkPYDelFreq);
+    else
+        fprintf((FILE *)a, "%s=%s\n", c->name, "CTRL_7");
+
+    return 0;
+}
+
 /* ƴԴʶּ */
 inline static int pinyin_get_word_from_phrase(Configure *c, void *a, int isread)
 {
@@ -677,6 +680,13 @@ Configure program_config[] = {
         .value_type = CONFIG_INTEGER,
         .value.integer = &bUseBold,
     },
+ #ifdef _ENABLE_TRAY
+    {
+        .name = "ʹͼ",
+        .value_type = CONFIG_INTEGER,
+        .value.integer = &bUseTrayIcon,
+    },
+ #endif
     {
         .name = NULL,
     },
@@ -909,6 +919,11 @@ Configure hotkey_config[] = {
         .name = "",
         .value_type = CONFIG_HOTKEY,
         .config_rw = hide_main_window,
+    },    
+    {
+        .name = "л",
+        .value_type = CONFIG_HOTKEY,
+        .config_rw = switch_vk,
     },
     {
         .name = "GBK֧",
@@ -958,6 +973,14 @@ Configure hotkey_config[] = {
     {
         .name = NULL,
     },
+    {
+        .name = "ʿ",
+        .value_type = CONFIG_HOTKEY,
+        .config_rw = save_all,
+    },
+    {
+        .name = NULL,
+    },
 };
 
 Configure input_method_config[] = {
@@ -1197,8 +1220,7 @@ void LoadConfig (Bool bMode)
         fp = fopen(buf, "r");
         if(!fp){
             perror("fopen");
-            exit(1);	/* װĿ¼ҲûļǾû취ˡ
-						 * ֻøû޷*/
+            exit(1);	// װĿ¼ҲûļǾֻøû޷
         }
     }
 
@@ -1274,11 +1296,11 @@ void LoadConfig (Bool bMode)
             fprintf(stderr, "error: configure file: no group name at beginning\n");
             exit(1);
         }
-
         //ҵе䱣浽Ӧȫֱȥ
         for(tmpconfig = configure_groups[group_idx].configure;
                 tmpconfig->name; tmpconfig++)
         {
+
             if(strncmp(tmpconfig->name, pbuf, pbuf1-pbuf) == 0)
                 read_configure(tmpconfig, ++pbuf1);
         }
@@ -1286,8 +1308,7 @@ void LoadConfig (Bool bMode)
 
     fclose(fp);
 
-    /* Ctrl+SpacefcitxĿؿݼ˴һṹá
-     * Ϊʲôڻȥܻߵ⡣*/
+    /* ļûô/ر뷨ȼôCTRL-SPACEΪĬȼ */
     if (!Trigger_Keys) {
 	iTriggerKeyCount = 0;
 	Trigger_Keys = (XIMTriggerKey *) malloc (sizeof (XIMTriggerKey) * (iTriggerKeyCount + 2));
@@ -1333,9 +1354,8 @@ void SaveConfig (void)
     for(tmpgroup = configure_groups; tmpgroup->name; tmpgroup++){
         if(tmpgroup->comment)
             fprintf(fp, "# %s\n", tmpgroup->comment);	// עͣд
-        fprintf(fp, "[%s]\n", tmpgroup->name);			// д
-        write_configures(fp, tmpgroup->configure);		/* 󽫸ÿ
-														 * д뵽ļ*/
+        fprintf(fp, "[%s]\n", tmpgroup->name);		// д
+        write_configures(fp, tmpgroup->configure);	// 󽫸ÿд뵽ļ
         fprintf(fp, "\n");		// ΪӿɶԲһ
     }
     fclose(fp);
@@ -1587,9 +1607,6 @@ void SaveProfile (void)
     iIMIndex_tmp = iIMIndex;		/* piaoairy add 20080518 */
     write_configures(fp, profiles);
     fclose(fp);
-
-    iTempInputWindowX = iInputWindowX;
-    iTempInputWindowY = iInputWindowY;
 }
 
 void SetHotKey (char *strKeys, HOTKEYS * hotkey)
