@@ -23,33 +23,33 @@
 
 #include <X11/Xlib.h>
 #include <iconv.h>
+#include <string.h>
 
 #include "xim.h"
 #include "IC.h"
 #include "tools.h"
 #include "MainWindow.h"
 #include "InputWindow.h"
+#ifdef _ENABLE_TRAY
+#include "TrayWindow.h"
+#endif
 #include "vk.h"
+#include "ui.h"
 
 CONNECT_ID     *connectIDsHead = (CONNECT_ID *) NULL;
+ICID	       *icidsHead = (ICID *) NULL;
 XIMS            ims;
 
 //************************************************
 CARD16          connect_id = 0;
-CARD16          icid = 0;
 CARD16          lastConnectID = 0;
-
 //************************************************
 
-long            filter_mask = KeyPressMask | KeyReleaseMask;
 IC             *CurrentIC = NULL;
 char            strLocale[201] = "zh_CN.GB18030,zh_CN.GB2312,zh_CN,zh_CN.GBK,zh_CN.UTF-8,zh_CN.UTF8,en_US.UTF-8,en_US.UTF8";
 
-//ñGTK+ OverTheSpotʱ
-INT8            iOffsetX = 0;
-INT8            iOffsetY = 16;
-
-//extern Bool     bLumaQQ;
+int		iClientCursorX = INPUTWND_STARTX;
+int		iClientCursorY = INPUTWND_STARTY;
 
 extern IM      *im;
 extern INT8     iIMIndex;
@@ -64,8 +64,6 @@ extern int      iMainWindowX;
 extern int      iMainWindowY;
 extern int      iInputWindowX;
 extern int      iInputWindowY;
-extern int      iTempInputWindowX;
-extern int      iTempInputWindowY;
 extern uint     iInputWindowHeight;
 extern uint     iInputWindowWidth;
 extern Bool     bTrackCursor;
@@ -76,7 +74,8 @@ extern iconv_t  convUTF8;
 extern uint     uMessageDown;
 extern uint     uMessageUp;
 extern Bool     bVK;
-extern Bool     bAutoHideInputWindow;
+extern Bool     bCorner;
+extern HIDE_MAINWINDOW hideMainWindow;
 
 //ٶ
 extern Bool     bStartRecordType;
@@ -86,37 +85,27 @@ extern Bool     bShowInputWindowTriggering;
 
 extern Bool     bUseGBKT;
 
-/*extern char	strUserLocale[];*/
-//+++++++++++++++++++++++++++++++++
-//extern INT8   iKeyPressed;
+#ifdef _DEBUG
+
+char            strUserLocale[100];
+char            strXModifiers[100];
+#endif
 
 static XIMStyle Styles[] = {
-//    XIMPreeditCallbacks | XIMStatusCallbacks, //OnTheSpot
-//    XIMPreeditCallbacks | XIMStatusArea,      //OnTheSpot
-//    XIMPreeditCallbacks | XIMStatusNothing,   //OnTheSpot
-    XIMPreeditPosition | XIMStatusArea,	//OverTheSpot
-    XIMPreeditPosition | XIMStatusNothing,	//OverTheSpot
-    XIMPreeditPosition | XIMStatusNone,	//OverTheSpot
-//    XIMPreeditArea | XIMStatusArea,           //OffTheSpot
-//    XIMPreeditArea | XIMStatusNothing,        //OffTheSpot
-//    XIMPreeditArea | XIMStatusNone,           //OffTheSpot
-    XIMPreeditNothing | XIMStatusNothing,	//Root
-    XIMPreeditNothing | XIMStatusNone,	//Root
+    XIMPreeditPosition | XIMStatusArea,		//OverTheSpot
+    XIMPreeditPosition | XIMStatusNothing,		//OverTheSpot
+    XIMPreeditPosition | XIMStatusNone,		//OverTheSpot
+    XIMPreeditNothing | XIMStatusNothing,		//Root
+    XIMPreeditNothing | XIMStatusNone,		//Root
+/*    XIMPreeditCallbacks | XIMStatusCallbacks,	//OnTheSpot
+    XIMPreeditCallbacks | XIMStatusArea,		//OnTheSpot
+    XIMPreeditCallbacks | XIMStatusNothing,		//OnTheSpot
+    XIMPreeditArea | XIMStatusArea,			//OffTheSpot
+    XIMPreeditArea | XIMStatusNothing,		//OffTheSpot
+    XIMPreeditArea | XIMStatusNone,			//OffTheSpot */
     0
 };
 
-/* Trigger Keys List */
-/*static XIMTriggerKey Trigger_Keys_Ctrl_Shift[] = {
-    {XK_space, ControlMask, ControlMask},
-    {XK_Shift_L, ControlMask, ControlMask},
-    {0L, 0L, 0L}
-};
-
-XIMTriggerKey   Trigger_Keys1[] = {
-    {XK_space, ControlMask, ControlMask},
-    {0L, 0L, 0L}
-};*/
-
 XIMTriggerKey  *Trigger_Keys = (XIMTriggerKey *) NULL;
 INT8            iTriggerKeyCount;
 
@@ -133,19 +122,12 @@ Bool MyOpenHandler (IMOpenStruct * call_data)
     return True;
 }
 
-Bool MyGetICValuesHandler (IMChangeICStruct * call_data)
-{
-    GetIC (call_data);
-
-    return True;
-}
-
-Bool MySetICValuesHandler (IMChangeICStruct * call_data)
+void SetTrackPos(IMChangeICStruct * call_data)
 {
     if (CurrentIC == NULL)
-	return True;
+	return;
     if (CurrentIC != (IC *) FindIC (call_data->icid))
-	return True;
+	return;
 
     if (bTrackCursor) {
 	int             i;
@@ -155,42 +137,32 @@ Bool MySetICValuesHandler (IMChangeICStruct * call_data)
 	for (i = 0; i < (int) ((IMChangeICStruct *) call_data)->preedit_attr_num; i++, pre_attr++) {
 	    if (!strcmp (XNSpotLocation, pre_attr->name)) {
 		if (CurrentIC->focus_win)
-		    XTranslateCoordinates (dpy, CurrentIC->focus_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iTempInputWindowX, &iTempInputWindowY, &window);
+		    XTranslateCoordinates (dpy, CurrentIC->focus_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iClientCursorX, &iClientCursorY, &window);
 		else if (CurrentIC->client_win)
-		    XTranslateCoordinates (dpy, CurrentIC->client_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iTempInputWindowX, &iTempInputWindowY, &window);
+		    XTranslateCoordinates (dpy, CurrentIC->client_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iClientCursorX, &iClientCursorY, &window);		    
 		else
-		    return True;
-
-		if (iTempInputWindowX < 0)
-		    iTempInputWindowX = 0;
-		else if ((iTempInputWindowX + iInputWindowWidth + iOffsetX) > DisplayWidth (dpy, iScreen))
-		    iTempInputWindowX = DisplayWidth (dpy, iScreen) - iInputWindowWidth;
-		else
-		    iTempInputWindowX += iOffsetX;
-
-		if (iTempInputWindowY < 0)
-		    iTempInputWindowY = 0;
-		else if ((iTempInputWindowY + iInputWindowHeight + iOffsetY) > DisplayHeight (dpy, iScreen))
-		    //iTempInputWindowY = DisplayHeight (dpy, iScreen) - 2 * iInputWindowHeight;
-		    iTempInputWindowY -= iInputWindowHeight;
-		else
-		    iTempInputWindowY += iOffsetY;
-
-		XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
-
+		    return;
+		    
 		ConnectIDSetTrackCursor (call_data->connect_id, True);
 	    }
 	}
     }
-    else if (bCenterInputWindow) {
-	iInputWindowX = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
-	if (iInputWindowX < 0)
-	    iInputWindowX = 0;
-	XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
-    }
 
+    MoveInputWindow(call_data->connect_id);
+}
+
+Bool MyGetICValuesHandler (IMChangeICStruct * call_data)
+{
+    GetIC (call_data);
+    
+    return True;
+}
+
+Bool MySetICValuesHandler (IMChangeICStruct * call_data)
+{
     SetIC (call_data);
-
+    SetTrackPos(call_data);
+    
     return True;
 }
 
@@ -198,56 +170,65 @@ Bool MySetFocusHandler (IMChangeFocusStruct * call_data)
 {
     CurrentIC = (IC *) FindIC (call_data->icid);
     connect_id = call_data->connect_id;
-    icid = call_data->icid;
 
     if (ConnectIDGetState (connect_id) != IS_CLOSED) {
-	IMPreeditStart (ims, (XPointer) call_data);
+	if (icidGetIMState(call_data->icid) == IS_CLOSED)
+	    IMPreeditStart (ims, (XPointer) call_data);
+ 	    
 	EnterChineseMode (lastConnectID == connect_id);
-
+	DrawMainWindow ();
+	
 	if (ConnectIDGetState (connect_id) == IS_CHN) {
 	    if (bVK)
 		DisplayVKWindow ();
-	    else if (lastConnectID != connect_id) {
-		if (!bAutoHideInputWindow)
-		    DisplayInputWindow ();
-		else
-		    XUnmapWindow (dpy, inputWindow);
-	    }
 	}
 	else {
 	    XUnmapWindow (dpy, inputWindow);
 	    XUnmapWindow (dpy, VKWindow);
-	}
-
-	if (ConnectIDGetTrackCursor (connect_id))
-	    XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
-	else
-	    XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
+	}	
     }
     else {
+	if (icidGetIMState(call_data->icid) != IS_CLOSED)
+	    IMPreeditEnd (ims, (XPointer) call_data);
+	    
 	XUnmapWindow (dpy, inputWindow);
 	XUnmapWindow (dpy, VKWindow);
-	DisplayMainWindow ();
+#ifdef _ENABLE_TRAY	
+	DrawTrayWindow (INACTIVE_ICON);
+#endif
+	if (hideMainWindow == HM_SHOW) {
+	    DisplayMainWindow ();
+	    DrawMainWindow ();
+	}
+	else 
+	    XUnmapWindow (dpy, mainWindow);
     }
 
+    icidSetIMState(call_data->icid, ConnectIDGetState (connect_id));
     lastConnectID = connect_id;
-    /*if (bLumaQQ && ConnectIDGetReset (connect_id)) {
-       SendHZtoClient ((IMForwardEventStruct *) call_data, "\0");
-       ConnectIDSetReset (connect_id, False);
-       } */
-
-    //When application gets the focus, rerecord the time.
+    //When application gets the focus, re-record the time.
     bStartRecordType = False;
     iHZInputed = 0;
-    DrawMainWindow ();
-
+   
+    if (ConnectIDGetTrackCursor (connect_id) && bTrackCursor) {
+	position * pos = ConnectIDGetPos(connect_id);
+	
+	if (pos) {
+	    iClientCursorX = pos->x;
+	    iClientCursorY = pos->y;
+	    XMoveWindow (dpy, inputWindow, iClientCursorX, iClientCursorY); 
+        }
+    }
+   
     return True;
 }
 
 Bool MyUnsetFocusHandler (IMChangeICStruct * call_data)
 {
-    XUnmapWindow (dpy, inputWindow);
-    XUnmapWindow (dpy, VKWindow);
+    if (call_data->connect_id==connect_id) {
+	XUnmapWindow (dpy, inputWindow);
+	XUnmapWindow (dpy, VKWindow);
+    }
 
     return True;
 }
@@ -255,11 +236,11 @@ Bool MyUnsetFocusHandler (IMChangeICStruct * call_data)
 Bool MyCloseHandler (IMOpenStruct * call_data)
 {
     XUnmapWindow (dpy, inputWindow);
-    XUnmapWindow (dpy, VKWindow);
+    XUnmapWindow (dpy, VKWindow);    
+    
     DestroyConnectID (call_data->connect_id);
     connect_id = 0;
-    icid = 0;
-
+    
     return True;
 }
 
@@ -268,11 +249,12 @@ Bool MyCreateICHandler (IMChangeICStruct * call_data)
     CreateIC (call_data);
 
     if (!CurrentIC) {
-	CurrentIC = (IC *) FindIC (call_data->icid);;
+	CurrentIC = (IC *) FindIC (call_data->icid);
 	connect_id = call_data->connect_id;
-	icid = call_data->icid;
     }
 
+    CreateICID(call_data);
+    
     return True;
 }
 
@@ -284,7 +266,8 @@ Bool MyDestroyICHandler (IMChangeICStruct * call_data)
     }
 
     DestroyIC (call_data);
-
+    DestroyICID(call_data->icid);
+    
     return True;
 }
 
@@ -299,28 +282,34 @@ void EnterChineseMode (Bool bState)
     }
 
     DisplayMainWindow ();
+#ifdef _ENABLE_TRAY	
+    DrawTrayWindow (ACTIVE_ICON);
+#endif
 }
 
 Bool MyTriggerNotifyHandler (IMTriggerNotifyStruct * call_data)
 {
-    if (call_data->flag == 0) {	/* on key */
-	/*
-	 * Here, the start of preediting is notified from IMlibrary, which
-	 * * is the only way to start preediting in case of Dynamic Event
-	 * * Flow, because ON key is mandatary for Dynamic Event Flow.
-	 */
+    if (call_data->flag == 0) {
+	/* Mainwindow always shows wrong input status, so fix it here */
+	CurrentIC = (IC *) FindIC (call_data->icid);
+        connect_id = call_data->connect_id;
+	    
 	SetConnectID (call_data->connect_id, IS_CHN);
+	icidSetIMState(call_data->icid, IS_CHN);
+	
 	EnterChineseMode (False);
 	DrawMainWindow ();
-
-	if (ConnectIDGetTrackCursor (connect_id))
-	    XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
-	else
-	    XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
-
-	if (bShowInputWindowTriggering)
+	
+	SetTrackPos( (IMChangeICStruct *)call_data );
+	if (bShowInputWindowTriggering && !bCorner)
 	    DisplayInputWindow ();
+	    
+#ifdef _ENABLE_TRAY	
+	DrawTrayWindow (ACTIVE_ICON);
+#endif
     }
+    else
+	return False;
 
     return True;
 }
@@ -330,72 +319,119 @@ Bool MyProtoHandler (XIMS _ims, IMProtocol * call_data)
     switch (call_data->major_code) {
     case XIM_OPEN:
 #ifdef _DEBUG
-	printf ("XIM_OPEN\n");
+        printf ("XIM_OPEN:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
-	return MyOpenHandler ((IMOpenStruct *) call_data);
+        return MyOpenHandler ((IMOpenStruct *) call_data);
     case XIM_CLOSE:
 #ifdef _DEBUG
-	printf ("XIM_CLOSE\n");
+        printf ("XIM_CLOSE:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyCloseHandler ((IMOpenStruct *) call_data);
     case XIM_CREATE_IC:
 #ifdef _DEBUG
-	printf ("XIM_CREATE_IC\n");
+        printf ("XIM_CREATE_IC:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyCreateICHandler ((IMChangeICStruct *) call_data);
     case XIM_DESTROY_IC:
 #ifdef _DEBUG
-	printf ("XIM_DESTROY_IC:%d\n", ((IMForwardEventStruct *) call_data)->icid);
+        printf ("XIM_DESTROY_IC:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyDestroyICHandler ((IMChangeICStruct *) call_data);
     case XIM_SET_IC_VALUES:
 #ifdef _DEBUG
-//      printf ("XIM_SET_IC_VALUES:%d\n", ((IMForwardEventStruct *) call_data)->icid);
+      printf ("XIM_SET_IC_VALUES:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MySetICValuesHandler ((IMChangeICStruct *) call_data);
     case XIM_GET_IC_VALUES:
 #ifdef _DEBUG
-	printf ("XIM_GET_IC_VALUES\n");
+        printf ("XIM_GET_IC_VALUES:\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyGetICValuesHandler ((IMChangeICStruct *) call_data);
     case XIM_FORWARD_EVENT:
 #ifdef _DEBUG
-	printf ("XIM_FORWARD_EVENT: %d  %d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
+        printf ("XIM_FORWARD_EVENT:\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
+	if (CurrentIC != FindIC (((IMChangeICStruct *)call_data)->icid)) {
+	    CurrentIC = FindIC (((IMChangeICStruct *)call_data)->icid);
+	    connect_id = ((IMChangeICStruct *)call_data)->connect_id;
+	    
+	    SetTrackPos((IMChangeICStruct *)call_data);
+        }
 	ProcessKey ((IMForwardEventStruct *) call_data);
+
 	return True;
     case XIM_SET_IC_FOCUS:
 #ifdef _DEBUG
-	printf ("XIM_SET_IC_FOCUS:conn=%d   ic=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
+	printf ("XIM_SET_IC_FOCUS:\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MySetFocusHandler ((IMChangeFocusStruct *) call_data);
     case XIM_UNSET_IC_FOCUS:
 #ifdef _DEBUG
-	printf ("XIM_UNSET_IC_FOCUS:%d\n", ((IMForwardEventStruct *) call_data)->icid);
+	printf ("XIM_UNSET_IC_FOCUS:\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyUnsetFocusHandler ((IMChangeICStruct *) call_data);;
     case XIM_RESET_IC:
 #ifdef _DEBUG
-	printf ("XIM_RESET_IC\n");
+        printf ("XIM_RESET_IC:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return True;
     case XIM_TRIGGER_NOTIFY:
 #ifdef _DEBUG
-	printf ("XIM_TRIGGER_NOTIFY\n");
+        printf ("XIM_TRIGGER_NOTIFY:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
 #endif
 	return MyTriggerNotifyHandler ((IMTriggerNotifyStruct *) call_data);
     default:
+#ifdef _DEBUG
+	printf ("XIM_DEFAULT:\t\ticid=%d\tconnect_id=%d\n", ((IMForwardEventStruct *) call_data)->icid, ((IMForwardEventStruct *) call_data)->connect_id);
+#endif
 	return True;
     }
 }
 
+void MyIMForwardEvent (CARD16 connectId, CARD16 icId, int keycode)
+{
+    IMForwardEventStruct forwardEvent;
+    XEvent          xEvent;
+
+    memset (&forwardEvent, 0, sizeof (IMForwardEventStruct));
+    forwardEvent.connect_id = connectId;
+    forwardEvent.icid = icId;
+    forwardEvent.major_code = XIM_FORWARD_EVENT;
+    forwardEvent.sync_bit = 0;
+    forwardEvent.serial_number = 0L;
+
+    xEvent.xkey.type = KeyPress;
+    xEvent.xkey.display = dpy;
+    xEvent.xkey.serial = 0L;
+    xEvent.xkey.send_event = False;
+    xEvent.xkey.x = xEvent.xkey.y = xEvent.xkey.x_root = xEvent.xkey.y_root = 0;
+    xEvent.xkey.same_screen = False;
+    xEvent.xkey.subwindow = None;
+    xEvent.xkey.window = None;
+    xEvent.xkey.root = DefaultRootWindow (dpy);
+    xEvent.xkey.state = 0;
+    if (CurrentIC->focus_win)
+	xEvent.xkey.window = CurrentIC->focus_win;
+    else if (CurrentIC->client_win)
+	xEvent.xkey.window = CurrentIC->client_win;
+
+    xEvent.xkey.keycode = keycode;
+    memcpy (&(forwardEvent.event), &xEvent, sizeof (forwardEvent.event));
+    IMForwardEvent (ims, (XPointer) (&forwardEvent));
+}
+
 void SendHZtoClient (IMForwardEventStruct * call_data, char *strHZ)
 {
     XTextProperty   tp;
+    IMCommitStruct  cms;
     char            strOutput[300];
     char           *ps;
     char           *pS2T = (char *) NULL;
 
+#ifdef _DEBUG
+    fprintf (stderr, "Sending %s  icid=%d connectid=%d\n", strHZ, CurrentIC->id, connect_id);
+#endif
+
     if (bUseGBKT)
 	pS2T = strHZ = ConvertGBKSimple2Tradition (strHZ);
 
@@ -413,36 +449,46 @@ void SendHZtoClient (IMForwardEventStruct * call_data, char *strHZ)
 	ps = strHZ;
 
     XmbTextListToTextProperty (dpy, (char **) &ps, 1, XCompoundTextStyle, &tp);
-    ((IMCommitStruct *) call_data)->flag |= XimLookupChars;
-    ((IMCommitStruct *) call_data)->commit_string = (char *) tp.value;
-    IMCommitString (ims, (XPointer) call_data);
+
+    memset (&cms, 0, sizeof (cms));
+    cms.major_code = XIM_COMMIT;
+    cms.icid = call_data->icid;
+    cms.connect_id = call_data->connect_id;
+    cms.flag = XimLookupChars;
+    cms.commit_string = (char *) tp.value;
+    IMCommitString (ims, (XPointer) & cms);
     XFree (tp.value);
+
     if (bUseGBKT)
 	free (pS2T);
 }
 
-Bool InitXIM (char *imname, Window im_window)
+Bool InitXIM (Window im_window, char *imname)
 {
     XIMStyles      *input_styles;
     XIMTriggerKeys *on_keys;
     XIMEncodings   *encodings;
     char           *p;
 
-    if (!imname) {
-	imname = getenv ("XMODIFIERS");
-	if (imname) {
-	    if (strstr (imname, "@im="))
-		imname += 4;
-	    else {
-		fprintf (stderr, "XMODIFIERS Error...\n");
-		imname = DEFAULT_IMNAME;
-	    }
-	}
-	else {
-	    fprintf (stderr, "Please set XMODIFIERS...\n");
-	    imname = DEFAULT_IMNAME;
-	}
+    if ( !imname ) {
+    	imname = getenv ("XMODIFIERS");
+    	if (imname) {
+            if (strstr (imname, "@im="))
+            	imname += 4;
+            else {
+                fprintf (stderr, "XMODIFIERS Error...\n");
+                imname = DEFAULT_IMNAME;
+            }
+        }
+        else {
+            fprintf (stderr, "Please set XMODIFIERS...\n");
+            imname = DEFAULT_IMNAME;
+        }
     }
+    
+#ifdef _DEBUG
+    strcpy (strXModifiers, imname);
+#endif
 
     input_styles = (XIMStyles *) malloc (sizeof (XIMStyles));
     input_styles->count_styles = sizeof (Styles) / sizeof (XIMStyle) - 1;
@@ -456,7 +502,6 @@ Bool InitXIM (char *imname, Window im_window)
     encodings->count_encodings = sizeof (zhEncodings) / sizeof (XIMEncoding) - 1;
     encodings->supported_encodings = zhEncodings;
 
-    /*ims = IMOpenIM (dpy, IMModifiers, "Xi18n", IMServerWindow, im_window, IMServerName, imname, IMLocale, (strUserLocale[0])? strUserLocale:"zh_CN.GB18030,zh_CN.GB2312,zh_CN,zh_CN.GBK,zh_CN.UTF-8,en_US.UTF-8", IMServerTransport, "X/", IMInputStyles, input_styles, NULL); */
     p = getenv ("LC_CTYPE");
     if (!p) {
 	p = getenv ("LC_ALL");
@@ -464,20 +509,25 @@ Bool InitXIM (char *imname, Window im_window)
 	    p = getenv ("LANG");
     }
     if (p) {
-	if (!strstr (strLocale, p)) {
+#ifdef _DEBUG
+	strcpy (strUserLocale, p);
+#endif
+	if (!strcasestr (strLocale, p)) {
 	    strcat (strLocale, ",");
 	    strcat (strLocale, p);
 	}
     }
 
     ims = IMOpenIM (dpy, IMModifiers, "Xi18n", IMServerWindow, im_window, IMServerName, imname, IMLocale, strLocale, IMServerTransport, "X/", IMInputStyles, input_styles, NULL);
+
     if (ims == (XIMS) NULL) {
 	fprintf (stderr, "Start FCITX error. Another XIM daemon named %s is running?\n", imname);
 	return False;
     }
 
+    IMSetIMValues (ims, IMEncodingList, encodings, IMProtocolHandler, MyProtoHandler, NULL);
+    IMSetIMValues (ims, IMFilterEventMask, KeyPressMask | KeyReleaseMask, NULL);
     IMSetIMValues (ims, IMOnKeysList, on_keys, NULL);
-    IMSetIMValues (ims, IMEncodingList, encodings, IMProtocolHandler, MyProtoHandler, IMFilterEventMask, filter_mask, NULL);
 
     return True;
 }
@@ -496,14 +546,11 @@ void SetIMState (Bool bState)
 	if (bState) {
 	    IMPreeditStart (ims, (XPointer) & call_data);
 	    SetConnectID (connect_id, IS_CHN);
-	    DrawMainWindow ();
 	}
 	else {
 	    IMPreeditEnd (ims, (XPointer) & call_data);
 	    SetConnectID (connect_id, IS_CLOSED);
 	    bVK = False;
-	    XUnmapWindow (dpy, inputWindow);
-	    XUnmapWindow (dpy, VKWindow);
 
 	    SwitchIM (-2);
 	}
@@ -512,28 +559,17 @@ void SetIMState (Bool bState)
 
 void CreateConnectID (IMOpenStruct * call_data)
 {
-    CONNECT_ID     *temp, *connectIDNew;
-
-    temp = connectIDsHead;
-    while (temp) {
-	if (!temp->next)
-	    break;
-	temp = temp->next;
-    }
+    CONNECT_ID     *connectIDNew;
 
     connectIDNew = (CONNECT_ID *) malloc (sizeof (CONNECT_ID));
-    connectIDNew->next = (CONNECT_ID *) NULL;
+    connectIDNew->next = connectIDsHead;
     connectIDNew->connect_id = call_data->connect_id;
     connectIDNew->imState = IS_CLOSED;
-    //connectIDNew->bReset = !bLumaQQ;
     connectIDNew->bTrackCursor = False;
-    //connectIDNew->strLocale=(char *)malloc(sizeof(char)*(call_data->lang.length+1));
-    //strcpy(connectIDNew->strLocale,call_data->lang.name);
+    connectIDNew->pos.x = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
+    connectIDNew->pos.y = DisplayHeight (dpy, iScreen) - iInputWindowHeight;
 
-    if (!temp)
-	connectIDsHead = connectIDNew;
-    else
-	temp->next = connectIDNew;
+    connectIDsHead = connectIDNew;
 }
 
 void DestroyConnectID (CARD16 connect_id)
@@ -556,9 +592,7 @@ void DestroyConnectID (CARD16 connect_id)
 	connectIDsHead = temp->next;
     else
 	last->next = temp->next;
-
-    /*if (temp->strLocale)
-       free(temp->strLocale); */
+	
     free (temp);
 }
 
@@ -593,7 +627,38 @@ IME_STATE ConnectIDGetState (CARD16 connect_id)
     return IS_CLOSED;
 }
 
-/*New Lumaqq need not be supported specially
+void ConnectIDSetPos (CARD16 connect_id, int x, int y)
+{
+    CONNECT_ID     *temp;
+
+    temp = connectIDsHead;
+    while (temp) {
+	if (temp->connect_id == connect_id) {
+	    temp->pos.x = x;
+	    temp->pos.y = y;
+	    return;
+	}
+
+	temp = temp->next;
+    }
+}
+
+position *ConnectIDGetPos (CARD16 connect_id)
+{
+    CONNECT_ID     *temp;
+
+    temp = connectIDsHead;
+    while (temp) {
+	if (temp->connect_id == connect_id)
+	    return &(temp->pos);
+
+	temp = temp->next;
+    }
+
+    return (position *)NULL;
+}
+
+/*
 Bool ConnectIDGetReset (CARD16 connect_id)
 {
     CONNECT_ID     *temp;
@@ -691,3 +756,92 @@ char *ConnectIDGetLocale(CARD16 connect_id)
     return strDefault;
 }
 */
+
+void CreateICID (IMChangeICStruct * call_data)
+{
+    ICID     *icidNew;
+
+    icidNew = (ICID *) malloc (sizeof (ICID));
+    icidNew->next = icidsHead;
+    icidNew->icid = call_data->icid;
+    icidNew->connect_id = call_data->connect_id;
+    icidNew->imState = IS_CLOSED;
+    
+    icidsHead = icidNew;
+}
+
+void DestroyICID (CARD16 icid)
+{
+    ICID     *temp, *last;
+
+    last = temp = icidsHead;
+    while (temp) {
+	if (temp->icid == icid)
+	    break;
+
+	last = temp;;
+	temp = temp->next;
+    }
+
+    if (!temp)
+	return;
+
+    if (temp == icidsHead)
+	icidsHead = temp->next;
+    else
+	last->next = temp->next;
+	
+    free (temp);
+}
+
+void icidSetIMState (CARD16 icid, IME_STATE imState)
+{
+    ICID     *temp;
+
+    temp = icidsHead;
+
+    while (temp) {
+	if (temp->icid == icid) {
+	    temp->imState = imState;
+#if _DEBUG
+	    fprintf(stderr,"Set icid=%d(connect_id=%d) to %d\n",icid,temp->connect_id, imState);
+#endif	    
+	    return;
+	}
+
+	temp = temp->next;
+    }
+}
+
+IME_STATE icidGetIMState (CARD16 icid)
+{
+    ICID     *temp;
+
+    temp = icidsHead;
+
+    while (temp) {
+	if (temp->icid == icid)
+	    return temp->imState;
+
+	temp = temp->next;
+    }
+
+    return IS_CLOSED;
+}
+/*
+CARD16 icidGetConnectID (CARD16 icid)
+{
+    ICID     *temp;
+
+    temp = icidsHead;
+
+    while (temp) {
+	if (temp->icid == icid)
+	    return temp->connect_id;
+
+	temp = temp->next;
+    }
+
+    return 0;
+}
+*/
